@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $superduper: input.c,v 1.9 2004/04/24 12:19:00 sam Exp $
+ * $superduper: input.c,v 1.13 2006/01/05 10:38:59 sam Exp $
  */
 #include "config.h"
 
@@ -57,6 +57,8 @@
 
 #include "input.h"
 
+#include "read_replaygain.h"
+
 #define DIR_SEPARATOR '/'
 #define HEADER_SIZE 256
 #define LINE_LENGTH 256
@@ -194,7 +196,7 @@ gboolean
 input_udp_init (struct mad_info_t *madinfo)
 {
   struct sockaddr_in sin;
-  int sinlen = sizeof (struct sockaddr_in);
+  unsigned int sinlen = sizeof (struct sockaddr_in);
 
   struct streamdata_t * sdata = madinfo->sdata;
   sdata->udpsock = socket (AF_INET, SOCK_DGRAM, 0);
@@ -319,7 +321,7 @@ input_http_init (struct mad_info_t *info)
         {
         }                       /* nothing */
       else
-        printf ("unknown header: %s\n", buffer);
+        g_message ("unknown header: %s", buffer);
     }
 
   info->title = g_strdup (info->sdata->name);
@@ -336,7 +338,7 @@ gboolean
 input_init (struct mad_info_t *info, const char *url)
 {
 #ifdef DEBUG
-  printf ("f: input_init\n");
+  g_message ("f: input_init");
 #endif
   memset (info, 0, sizeof (struct mad_info_t));
 
@@ -356,6 +358,14 @@ input_init (struct mad_info_t *info, const char *url)
   info->mode          = 0;
   info->title         = 0;
   info->sdata         = streamdata_new ();
+  info->offset        = 0;
+
+  info->replaygain_album_str = 0;
+  info->replaygain_track_str = 0;
+  info->replaygain_album_peak_str = 0;
+  info->replaygain_track_peak_str = 0;
+  info->mp3gain_undo_str = 0;
+  info->mp3gain_minmax_str = 0;
 
   input_parse_url (info);
 
@@ -378,7 +388,7 @@ input_init (struct mad_info_t *info, const char *url)
       info->size = buf.st_size;
     }
 #ifdef DEBUG
-  printf ("e: input_init\n");
+  g_message ("e: input_init");
 #endif
   return TRUE;
 }
@@ -387,7 +397,7 @@ input_init (struct mad_info_t *info, const char *url)
 gchar* 
 input_id3_get_string (struct id3_tag *tag, char *frame_name)
 {
-  gchar* rtn;
+  guchar* rtn;
   const id3_ucs4_t *string;
   struct id3_frame *frame;
   union id3_field *field;
@@ -413,7 +423,7 @@ input_id3_get_string (struct id3_tag *tag, char *frame_name)
      string = id3_genre_name (string);
 
   rtn = id3_ucs4_latin1duplicate (string);
-  return rtn;
+  return (gchar*)rtn;
 }
 
 /**
@@ -432,13 +442,21 @@ input_read_tag (struct mad_info_t *info)
   info->tag = id3_file_tag (info->id3file);
   if (!info->tag) return;
 
-  title_input->performer = input_id3_get_string (info->tag, ID3_FRAME_ARTIST);
+  title_input->performer  = input_id3_get_string (info->tag, ID3_FRAME_ARTIST);
   title_input->track_name = input_id3_get_string (info->tag, ID3_FRAME_TITLE);
   title_input->album_name = input_id3_get_string (info->tag, ID3_FRAME_ALBUM);
-  title_input->genre = input_id3_get_string (info->tag, ID3_FRAME_GENRE);
-  title_input->comment = input_id3_get_string (info->tag, ID3_FRAME_COMMENT);
+  title_input->genre      = input_id3_get_string (info->tag, ID3_FRAME_GENRE);
+  title_input->comment    = input_id3_get_string (info->tag, ID3_FRAME_COMMENT);
+  title_input->date       = input_id3_get_string (info->tag, ID3_FRAME_YEAR);
+  title_input->file_name  = g_strdup(g_basename(info->filename));
+  title_input->file_path  = g_dirname(info->filename);
+  if ((string = strrchr (title_input->file_name, '.')))
+    {
+      title_input->file_ext = string+1;
+      *string= '\0'; // make filename end at dot.
+    }
   string = input_id3_get_string (info->tag, ID3_FRAME_TRACK);
-  if (string) 
+  if (string)
     {
       title_input->track_number = atoi (string);
       g_free (string);
@@ -450,6 +468,9 @@ input_read_tag (struct mad_info_t *info)
   if (title_input->album_name)   g_free (title_input->album_name);
   if (title_input->genre)        g_free (title_input->genre);
   if (title_input->comment)      g_free (title_input->comment);
+  if (title_input->date)         g_free (title_input->date);
+  if (title_input->file_name)    g_free (title_input->file_name);
+  if (title_input->file_path)    g_free (title_input->file_path);
   g_free (title_input);
 }
 
@@ -473,22 +494,24 @@ input_get_info (struct mad_info_t *info, gboolean fast_scan)
   else
     { /* local mp3 file */
       input_read_tag (info);
+      input_read_replaygain(info);
       /* scan mp3 file, decoding headers unless fast_scan is set */
       if (scan_file (info, fast_scan) == FALSE)
-	{ 
+	{
 	  return FALSE;
 	}
       /* reset the input file to the start */
       lseek (info->infile, 0, SEEK_SET);
+      info->offset = 0;
       /* use the filename for the title as a last resort */
       if (!info->title) 
-        {
-          char *pos = strrchr (info->filename, DIR_SEPARATOR);
-          if (pos)
-            info->title = g_strdup (pos + 1);
-          else
-            info->title = g_strdup (info->filename);
-        }
+	{
+	  char *pos = strrchr (info->filename, DIR_SEPARATOR);
+	  if (pos)
+	    info->title = g_strdup (pos + 1);
+	  else
+	    info->title = g_strdup (info->filename);
+	}
     }
 #ifdef DEBUG
   g_message ("e: input_get_info");
@@ -563,6 +586,7 @@ input_rebuffer (struct mad_info_t *madinfo)
       if (len == -1)
         return -1;
       madinfo->sdata->b_buffer_end += len;
+      madinfo->offset += len;
       data_read += len;
     }
 
@@ -579,7 +603,7 @@ input_rebuffer (struct mad_info_t *madinfo)
  * @return -1 on error
  */
 int
-input_get_data (struct mad_info_t *madinfo, char *buffer, int buffer_size)
+input_get_data (struct mad_info_t *madinfo, guchar *buffer, int buffer_size)
 {
   int len = 0;
   int remainder = 0;
@@ -641,8 +665,9 @@ input_get_data (struct mad_info_t *madinfo, char *buffer, int buffer_size)
       len = read (madinfo->infile, buffer, buffer_size);
     }
 #ifdef DEBUG
-  g_message ("e: input_get_data: %d", len);
+  g_message ("e: input_get_data: size=%d offset=%d", len, madinfo->offset);
 #endif
+  madinfo->offset += len;
   return len;
 }
 
@@ -663,6 +688,13 @@ input_term (struct mad_info_t *info)
   if (info->sdata   ) streamdata_free (info->sdata);
   if (info->id3file ) id3_file_close (info->id3file);
 
+  if (info->replaygain_album_str ) g_free(info->replaygain_album_str);
+  if (info->replaygain_track_str ) g_free(info->replaygain_track_str);
+  if (info->replaygain_album_peak_str ) g_free(info->replaygain_album_peak_str);
+  if (info->replaygain_track_peak_str ) g_free(info->replaygain_track_peak_str);
+  if (info->mp3gain_undo_str) g_free(info->mp3gain_undo_str);
+  if (info->mp3gain_minmax_str) g_free(info->mp3gain_minmax_str);
+
   /* set everything to zero in case it gets used again. */
   memset(info, 0, sizeof(struct mad_info_t));
 #ifdef DEBUG
