@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: xmms-mad.c,v 1.18 2006/01/05 10:38:59 sam Exp $
+ * $Id: xmms-mad.c,v 1.19 2006/01/12 17:31:28 sam Exp $
  */
 #include "config.h"
 #include "xmms-mad.h"
@@ -113,28 +113,40 @@ xmmsmad_set_eq (int on, float preamp, float *bands)
   //printf("set_eq\n");
 }
 
-static int mpg123_head_check(unsigned long head)
+static int mp3_head_check(const guchar* head)
 {
     /*
      * First two bytes must be a sync header (11 bits all 1)
      * http://www.mp3-tech.org/programmer/frame_header.html
      */
-    if ((head & 0xffe00000) != 0xffe00000)
+    if (head[0] != 0xff || (head[1] & 0xe0) != 0xe0)
       return FALSE;
-    if (!((head >> 17) & 3))
+
+    /* These check come from mgp123_head_check */
+
+    if ((head[1] & 0x0c) == 0x00)
+      /* bits 18 and 19 are not set */
       return FALSE;
-    if (((head >> 12) & 0xf) == 0xf)
+
+    if ((head[2] & 0xf0) == 0xf0)
+      /* bits 13 - 16 are all set */
       return FALSE;
-    if (!((head >> 12) & 0xf))
+
+    if ((head[2] & 0xf0) == 0x00)
+      /* bits 13 - 16 are all not set */
       return FALSE;
-    if (((head >> 10) & 0x3) == 0x3)
+
+    if (((head[2] >> 2) & 0x3) == 0x3)
+      /* bit 11 and 12 are both set */
       return FALSE;
-    if (((head >> 19) & 1) == 1 &&
-        ((head >> 17) & 3) == 3 &&
-        ((head >> 16) & 1) == 1)
+
+    if ((head[1] & 0x17) == 0x17)
+      /* bits 17 - 20 are all set */
       return FALSE;
-    if ((head & 0xffff0000) == 0xfffe0000)
+
+    if ((head[1] & 0xff) == 0xfe)
       return FALSE;
+
     return TRUE;
 }
 
@@ -167,7 +179,7 @@ xmmsmad_is_our_file (char *filename)
           /*
 	   * or three bytes are "ID3"
 	   */
-	  if (mpg123_head_check(*(unsigned long*)check))
+	  if (mp3_head_check(check))
 	    {
 	      rtn = 1;
 	    }
@@ -187,7 +199,7 @@ xmmsmad_is_our_file (char *filename)
 #if 0
 	      /* FIXME: make this test endian safe */
 	      else if (memcmp (check, "WAVE"))
-	        {
+		{
 		  /* seek to codec ID and see if it's MP3 */
 		  lseek (fin, 20, SEEK_SET);
 		  check = 0L;
